# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_type import EntityType
from .scoped_entity import ScopedEntity


class KnowledgeContextFilter_ByEntities(UniversalBaseModel):
    scope_type: typing_extensions.Annotated[typing.Literal["byEntities"], FieldMetadata(alias="scopeType")] = (
        "byEntities"
    )
    entities: typing.List[ScopedEntity]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class KnowledgeContextFilter_ByEntityTypes(UniversalBaseModel):
    scope_type: typing_extensions.Annotated[typing.Literal["byEntityTypes"], FieldMetadata(alias="scopeType")] = (
        "byEntityTypes"
    )
    entity_types: typing_extensions.Annotated[typing.Set[EntityType], FieldMetadata(alias="entityTypes")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


KnowledgeContextFilter = typing.Union[KnowledgeContextFilter_ByEntities, KnowledgeContextFilter_ByEntityTypes]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .customer_status import CustomerStatus


class CustomerBase(UniversalBaseModel):
    name: str
    description: typing.Optional[str] = None
    metadata: typing.Dict[str, str]
    status: CustomerStatus = pydantic.Field()
    """
    Whether or not the customer is in active use.
    
    Only active customers will be available for agent interactions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .customer_base import CustomerBase
from .customer_field import CustomerField
from .customer_filter import CustomerFilter
from .customer_patch_request import CustomerPatchRequest
from .customer_request import CustomerRequest
from .customer_response import CustomerResponse
from .customer_status import CustomerStatus
from .customers_search_request import CustomersSearchRequest
from .customers_search_response import CustomersSearchResponse

__all__ = [
    "CustomerBase",
    "CustomerField",
    "CustomerFilter",
    "CustomerPatchRequest",
    "CustomerRequest",
    "CustomerResponse",
    "CustomerStatus",
    "CustomersSearchRequest",
    "CustomersSearchResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .harmful_content_analysis import HarmfulContentAnalysis


class SafetyCheckReport(UniversalBaseModel):
    attack_detected: typing_extensions.Annotated[bool, FieldMetadata(alias="attackDetected")] = pydantic.Field()
    """
    Whether the check detected a malicious attack.
    """

    harmful_content_detected: typing_extensions.Annotated[bool, FieldMetadata(alias="harmfulContentDetected")] = (
        pydantic.Field()
    )
    """
    Whether the check detected harmful content such as hate speech, references to self-harm, violence or sexual references.
    """

    harmful_content_analysis: typing_extensions.Annotated[
        typing.List[HarmfulContentAnalysis], FieldMetadata(alias="harmfulContentAnalysis")
    ] = pydantic.Field()
    """
    Includes details on each category of harmful content.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.entity_id_base import EntityIdBase
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawCustomersClient, RawCustomersClient
from .types.customer_field import CustomerField
from .types.customer_filter import CustomerFilter
from .types.customer_response import CustomerResponse
from .types.customer_status import CustomerStatus
from .types.customers_search_response import CustomersSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomersClient
        """
        return self._raw_client

    def search(
        self,
        *,
        sort: typing.Optional[CustomerField] = OMIT,
        filter: typing.Optional[CustomerFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CustomersSearchResponse:
        """
        Parameters
        ----------
        sort : typing.Optional[CustomerField]
            The field to sort by, defaults to created timestamp

        filter : typing.Optional[CustomerFilter]
            The filter to apply to the customers.

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomersSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.customers.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def create_or_update(
        self, *, customer_id: EntityIdBase, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CustomerResponse:
        """
        Update a customer of an agent or create it if it doesn't exist. In case of an update, fields not provided (e.g., description, status) will be preserved.

        Parameters
        ----------
        customer_id : EntityIdBase
            ID that uniquely identifies this customer

        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.customers.create_or_update(
            customer_id=EntityIdBase(
                reference_id="acme",
            ),
            name="Acme Corporation",
        )
        """
        _response = self._raw_client.create_or_update(
            customer_id=customer_id, name=name, request_options=request_options
        )
        return _response.data

    def get(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CustomerResponse:
        """
        Get a customer by its supplied ID

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to get. If not provided, the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.customers.get(
            customer_reference_id="acme",
        )
        """
        _response = self._raw_client.get(customer_reference_id, app_id=app_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[CustomerStatus] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        assignees: typing.Optional[typing.Set[str]] = OMIT,
        members: typing.Optional[typing.Set[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CustomerResponse:
        """
        Update mutable customer fields

        The `appId` field can be provided to update a customer owned by a different app.
        All other fields will overwrite the existing value on the customer only if provided.

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to update. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to update. If not provided, the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the customer.

        description : typing.Optional[str]
            The description of the customer.

        status : typing.Optional[CustomerStatus]
            Whether or not the customer is in active use.

            Only active customers will be available for agent interactions.

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata associated with the customer. If not provided, the existing metadata will be preserved.

        assignees : typing.Optional[typing.Set[str]]
            Set of agent user IDs to assign to this customer. If provided, replaces all existing assignees.

        members : typing.Optional[typing.Set[str]]
            Set of agent user IDs who are members of this customer. If provided, replaces all existing members.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerResponse
            The updated customer

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.customers.patch(
            customer_reference_id="customerReferenceId",
        )
        """
        _response = self._raw_client.patch(
            customer_reference_id,
            app_id=app_id,
            name=name,
            description=description,
            status=status,
            metadata=metadata,
            assignees=assignees,
            members=members,
            request_options=request_options,
        )
        return _response.data


class AsyncCustomersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomersClient
        """
        return self._raw_client

    async def search(
        self,
        *,
        sort: typing.Optional[CustomerField] = OMIT,
        filter: typing.Optional[CustomerFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CustomersSearchResponse:
        """
        Parameters
        ----------
        sort : typing.Optional[CustomerField]
            The field to sort by, defaults to created timestamp

        filter : typing.Optional[CustomerFilter]
            The filter to apply to the customers.

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomersSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.customers.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def create_or_update(
        self, *, customer_id: EntityIdBase, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CustomerResponse:
        """
        Update a customer of an agent or create it if it doesn't exist. In case of an update, fields not provided (e.g., description, status) will be preserved.

        Parameters
        ----------
        customer_id : EntityIdBase
            ID that uniquely identifies this customer

        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.customers.create_or_update(
                customer_id=EntityIdBase(
                    reference_id="acme",
                ),
                name="Acme Corporation",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(
            customer_id=customer_id, name=name, request_options=request_options
        )
        return _response.data

    async def get(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CustomerResponse:
        """
        Get a customer by its supplied ID

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to get. If not provided, the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.customers.get(
                customer_reference_id="acme",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(customer_reference_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def patch(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[CustomerStatus] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        assignees: typing.Optional[typing.Set[str]] = OMIT,
        members: typing.Optional[typing.Set[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CustomerResponse:
        """
        Update mutable customer fields

        The `appId` field can be provided to update a customer owned by a different app.
        All other fields will overwrite the existing value on the customer only if provided.

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to update. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to update. If not provided, the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the customer.

        description : typing.Optional[str]
            The description of the customer.

        status : typing.Optional[CustomerStatus]
            Whether or not the customer is in active use.

            Only active customers will be available for agent interactions.

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata associated with the customer. If not provided, the existing metadata will be preserved.

        assignees : typing.Optional[typing.Set[str]]
            Set of agent user IDs to assign to this customer. If provided, replaces all existing assignees.

        members : typing.Optional[typing.Set[str]]
            Set of agent user IDs who are members of this customer. If provided, replaces all existing members.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CustomerResponse
            The updated customer

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.customers.patch(
                customer_reference_id="customerReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            customer_reference_id,
            app_id=app_id,
            name=name,
            description=description,
            status=status,
            metadata=metadata,
            assignees=assignees,
            members=members,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.error_message import ErrorMessage
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.customer_field import CustomerField
from .types.customer_filter import CustomerFilter
from .types.customer_response import CustomerResponse
from .types.customer_status import CustomerStatus
from .types.customers_search_response import CustomersSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search(
        self,
        *,
        sort: typing.Optional[CustomerField] = OMIT,
        filter: typing.Optional[CustomerFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CustomersSearchResponse]:
        """
        Parameters
        ----------
        sort : typing.Optional[CustomerField]
            The field to sort by, defaults to created timestamp

        filter : typing.Optional[CustomerFilter]
            The filter to apply to the customers.

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CustomersSearchResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/customers",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=CustomerFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomersSearchResponse,
                    parse_obj_as(
                        type_=CustomersSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_or_update(
        self, *, customer_id: EntityIdBase, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CustomerResponse]:
        """
        Update a customer of an agent or create it if it doesn't exist. In case of an update, fields not provided (e.g., description, status) will be preserved.

        Parameters
        ----------
        customer_id : EntityIdBase
            ID that uniquely identifies this customer

        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CustomerResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/customers",
            method="PUT",
            json={
                "customerId": convert_and_respect_annotation_metadata(
                    object_=customer_id, annotation=EntityIdBase, direction="write"
                ),
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerResponse,
                    parse_obj_as(
                        type_=CustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CustomerResponse]:
        """
        Get a customer by its supplied ID

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to get. If not provided, the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CustomerResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/customers/{jsonable_encoder(customer_reference_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerResponse,
                    parse_obj_as(
                        type_=CustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[CustomerStatus] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        assignees: typing.Optional[typing.Set[str]] = OMIT,
        members: typing.Optional[typing.Set[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CustomerResponse]:
        """
        Update mutable customer fields

        The `appId` field can be provided to update a customer owned by a different app.
        All other fields will overwrite the existing value on the customer only if provided.

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to update. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to update. If not provided, the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the customer.

        description : typing.Optional[str]
            The description of the customer.

        status : typing.Optional[CustomerStatus]
            Whether or not the customer is in active use.

            Only active customers will be available for agent interactions.

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata associated with the customer. If not provided, the existing metadata will be preserved.

        assignees : typing.Optional[typing.Set[str]]
            Set of agent user IDs to assign to this customer. If provided, replaces all existing assignees.

        members : typing.Optional[typing.Set[str]]
            Set of agent user IDs who are members of this customer. If provided, replaces all existing members.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CustomerResponse]
            The updated customer
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/customers/{jsonable_encoder(customer_reference_id)}",
            method="PATCH",
            json={
                "appId": app_id,
                "name": name,
                "description": description,
                "status": status,
                "metadata": metadata,
                "assignees": assignees,
                "members": members,
            },
            headers={
                "content-type": "application/merge-patch+json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerResponse,
                    parse_obj_as(
                        type_=CustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCustomersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search(
        self,
        *,
        sort: typing.Optional[CustomerField] = OMIT,
        filter: typing.Optional[CustomerFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CustomersSearchResponse]:
        """
        Parameters
        ----------
        sort : typing.Optional[CustomerField]
            The field to sort by, defaults to created timestamp

        filter : typing.Optional[CustomerFilter]
            The filter to apply to the customers.

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CustomersSearchResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/customers",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=CustomerFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomersSearchResponse,
                    parse_obj_as(
                        type_=CustomersSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_or_update(
        self, *, customer_id: EntityIdBase, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CustomerResponse]:
        """
        Update a customer of an agent or create it if it doesn't exist. In case of an update, fields not provided (e.g., description, status) will be preserved.

        Parameters
        ----------
        customer_id : EntityIdBase
            ID that uniquely identifies this customer

        name : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CustomerResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/customers",
            method="PUT",
            json={
                "customerId": convert_and_respect_annotation_metadata(
                    object_=customer_id, annotation=EntityIdBase, direction="write"
                ),
                "name": name,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerResponse,
                    parse_obj_as(
                        type_=CustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CustomerResponse]:
        """
        Get a customer by its supplied ID

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to get. If not provided, the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CustomerResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/customers/{jsonable_encoder(customer_reference_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerResponse,
                    parse_obj_as(
                        type_=CustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch(
        self,
        customer_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        status: typing.Optional[CustomerStatus] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        assignees: typing.Optional[typing.Set[str]] = OMIT,
        members: typing.Optional[typing.Set[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CustomerResponse]:
        """
        Update mutable customer fields

        The `appId` field can be provided to update a customer owned by a different app.
        All other fields will overwrite the existing value on the customer only if provided.

        Parameters
        ----------
        customer_reference_id : str
            The reference ID of the customer to update. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the customer to update. If not provided, the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the customer.

        description : typing.Optional[str]
            The description of the customer.

        status : typing.Optional[CustomerStatus]
            Whether or not the customer is in active use.

            Only active customers will be available for agent interactions.

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata associated with the customer. If not provided, the existing metadata will be preserved.

        assignees : typing.Optional[typing.Set[str]]
            Set of agent user IDs to assign to this customer. If provided, replaces all existing assignees.

        members : typing.Optional[typing.Set[str]]
            Set of agent user IDs who are members of this customer. If provided, replaces all existing members.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CustomerResponse]
            The updated customer
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/customers/{jsonable_encoder(customer_reference_id)}",
            method="PATCH",
            json={
                "appId": app_id,
                "name": name,
                "description": description,
                "status": status,
                "metadata": metadata,
                "assignees": assignees,
                "members": members,
            },
            headers={
                "content-type": "application/merge-patch+json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerResponse,
                    parse_obj_as(
                        type_=CustomerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .bot_logic_action_executed_detail import BotLogicActionExecutedDetail
from .bot_logic_action_reviewed_detail import BotLogicActionReviewedDetail


class BotLogicActionItem(UniversalBaseModel):
    actions_reviewed: typing_extensions.Annotated[
        typing.List[BotLogicActionReviewedDetail], FieldMetadata(alias="actionsReviewed")
    ]
    actions_executed: typing_extensions.Annotated[
        typing.List[BotLogicActionExecutedDetail], FieldMetadata(alias="actionsExecuted")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.event_filter import EventFilter
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .event_group_by import EventGroupBy
from .event_metric import EventMetric
from .time_interval import TimeInterval


class EventChartRequest_PieChart(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import (
        EventChartRequest_PieChart,
        EventGroupBy,
        EventMetric_Count,
    )
    from mavenagi.commons import EventFilter

    EventChartRequest_PieChart(
        event_filter=EventFilter(
            event_types=["USER"],
        ),
        group_by=EventGroupBy(
            field="EVENT_NAME",
        ),
        metric=EventMetric_Count(),
    )
    """

    type: typing.Literal["pieChart"] = "pieChart"
    group_by: typing_extensions.Annotated[EventGroupBy, FieldMetadata(alias="groupBy")]
    metric: EventMetric
    event_filter: typing_extensions.Annotated[typing.Optional[EventFilter], FieldMetadata(alias="eventFilter")] = None
    timezone: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EventChartRequest_DateHistogram(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import (
        EventChartRequest_PieChart,
        EventGroupBy,
        EventMetric_Count,
    )
    from mavenagi.commons import EventFilter

    EventChartRequest_PieChart(
        event_filter=EventFilter(
            event_types=["USER"],
        ),
        group_by=EventGroupBy(
            field="EVENT_NAME",
        ),
        metric=EventMetric_Count(),
    )
    """

    type: typing.Literal["dateHistogram"] = "dateHistogram"
    time_interval: typing_extensions.Annotated[TimeInterval, FieldMetadata(alias="timeInterval")]
    group_by: typing_extensions.Annotated[typing.Optional[EventGroupBy], FieldMetadata(alias="groupBy")] = None
    metric: EventMetric
    event_filter: typing_extensions.Annotated[typing.Optional[EventFilter], FieldMetadata(alias="eventFilter")] = None
    timezone: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EventChartRequest_BarChart(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import (
        EventChartRequest_PieChart,
        EventGroupBy,
        EventMetric_Count,
    )
    from mavenagi.commons import EventFilter

    EventChartRequest_PieChart(
        event_filter=EventFilter(
            event_types=["USER"],
        ),
        group_by=EventGroupBy(
            field="EVENT_NAME",
        ),
        metric=EventMetric_Count(),
    )
    """

    type: typing.Literal["barChart"] = "barChart"
    bar_definition: typing_extensions.Annotated[EventGroupBy, FieldMetadata(alias="barDefinition")]
    metric: EventMetric
    vertical_grouping: typing_extensions.Annotated[
        typing.Optional[EventGroupBy], FieldMetadata(alias="verticalGrouping")
    ] = None
    event_filter: typing_extensions.Annotated[typing.Optional[EventFilter], FieldMetadata(alias="eventFilter")] = None
    timezone: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mavenagi.analytics import (
    EventChartRequest_PieChart,
    EventGroupBy,
    EventMetric_Count,
)
from mavenagi.commons import EventFilter

EventChartRequest_PieChart(
    event_filter=EventFilter(
        event_types=["USER"],
    ),
    group_by=EventGroupBy(
        field="EVENT_NAME",
    ),
    metric=EventMetric_Count(),
)
"""
EventChartRequest = typing.Union[
    EventChartRequest_PieChart, EventChartRequest_DateHistogram, EventChartRequest_BarChart
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .customer_status import CustomerStatus


class CustomerFilter(UniversalBaseModel):
    search: typing.Optional[str] = pydantic.Field(default=None)
    """
    The search query to apply to the customers' name and description.
    """

    statuses: typing.Optional[typing.List[CustomerStatus]] = pydantic.Field(default=None)
    """
    The statuses to filter the customers by.
    """

    assignees: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter customers by assignee agent user IDs. Returns customers that have any of the specified assignees.
    """

    members: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter customers by member agent user IDs. Returns customers that have any of the specified members.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class HarmfulContentAnalysis(UniversalBaseModel):
    category: str = pydantic.Field()
    """
    The category of harmful content analysis.
    """

    harmful_content_detected: typing_extensions.Annotated[bool, FieldMetadata(alias="harmfulContentDetected")] = (
        pydantic.Field()
    )
    """
    Whether the analysis detected harmful content for this category. Will be true if the severity is greater than or equal to the threshold.
    """

    severity: int = pydantic.Field()
    """
    The severity of the harmful content analysis. Will be 0 if the category was not detected.
    """

    threshold: int = pydantic.Field()
    """
    The threshold for the content safety analysis. If the severity is greater than or equal to the threshold, the analysis will be considered harmful.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.event_field import EventField
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class EventMetric_Count(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Currently only count metric is supported for events.
    """

    type: typing.Literal["count"] = "count"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EventMetric_DistinctCount(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Currently only count metric is supported for events.
    """

    type: typing.Literal["distinctCount"] = "distinctCount"
    target_field: typing_extensions.Annotated[EventField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


EventMetric = typing.Union[EventMetric_Count, EventMetric_DistinctCount]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .customer_status import CustomerStatus


class CustomerPatchRequest(UniversalBaseModel):
    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = pydantic.Field(
        default=None
    )
    """
    The App ID of the customer to update. If not provided, the ID of the calling app will be used.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the customer.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the customer.
    """

    status: typing.Optional[CustomerStatus] = pydantic.Field(default=None)
    """
    Whether or not the customer is in active use.
    
    Only active customers will be available for agent interactions.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata associated with the customer. If not provided, the existing metadata will be preserved.
    """

    assignees: typing.Optional[typing.Set[str]] = pydantic.Field(default=None)
    """
    Set of agent user IDs to assign to this customer. If provided, replaces all existing assignees.
    """

    members: typing.Optional[typing.Set[str]] = pydantic.Field(default=None)
    """
    Set of agent user IDs who are members of this customer. If provided, replaces all existing members.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
